'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var PropTypes = require('prop-types');
var FileSaver = require('file-saver');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefault(React);
var PropTypes__default = /*#__PURE__*/_interopDefault(PropTypes);
var FileSaver__default = /*#__PURE__*/_interopDefault(FileSaver);

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

/* eslint no-unneeded-ternary: 0 */
const getMetaInfo = columns => columns.map(column => ({
  field: column.dataField,
  type: column.csvType || String,
  formatter: column.csvFormatter,
  formatExtraData: column.formatExtraData,
  header: column.csvText || column.text,
  export: column.csvExport === false ? false : true,
  row: Number(column.row) || 0,
  rowSpan: Number(column.rowSpan) || 1,
  colSpan: Number(column.colSpan) || 1,
  footer: column.footer,
  footerFormatter: column.footerFormatter
})).filter(_ => _.export);
const transform = (data, meta, columns, _, {
  separator,
  ignoreHeader,
  ignoreFooter
}) => {
  const visibleColumns = meta.filter(m => m.export);
  let content = ''; // extract csv header

  if (!ignoreHeader) {
    content += visibleColumns.map(m => `"${m.header}"`).join(separator);
    content += '\n';
  } // extract csv body


  if (data.length === 0) return content;
  content += data.map((row, rowIndex) => visibleColumns.map(m => {
    let cellContent = _.get(row, m.field);

    if (m.formatter) {
      cellContent = m.formatter(cellContent, row, rowIndex, m.formatExtraData);
    }

    if (m.type === String) {
      return `"${`${cellContent}`.replace(/"/g, '""')}"`;
    }

    return cellContent;
  }).join(separator)).join('\n');

  if (!ignoreFooter) {
    content += '\n';
    content += visibleColumns.map((m, i) => {
      if (typeof m.footer === 'function') {
        const columnData = _.pluck(data, columns[i].dataField);

        return `"${m.footer(columnData, columns[i], i)}"`;
      }

      if (m.footerFormatter) {
        return `"${m.footerFormatter(columns[i], i)}"`;
      }

      return `"${m.footer}"`;
    }).join(separator);
  }

  return content;
};
const save = (content, {
  noAutoBOM,
  fileName,
  blobType
}) => {
  FileSaver__default['default'].saveAs(new Blob([content], {
    type: blobType
  }), fileName, noAutoBOM);
};

const csvDefaultOptions = {
  fileName: 'spreadsheet.csv',
  separator: ',',
  ignoreHeader: false,
  ignoreFooter: true,
  noAutoBOM: true,
  blobType: 'text/plain;charset=utf-8',
  exportAll: true,
  onlyExportSelection: false
};
var csvOperation = (Base => {
  var _temp;

  return _temp = class CSVOperation extends Base {
    constructor(...args) {
      super(...args);

      this.handleExportCSV = source => {
        const {
          columns,
          exportCSV,
          keyField
        } = this.props;
        const meta = getMetaInfo(columns);
        const options = exportCSV === true ? csvDefaultOptions : _objectSpread2(_objectSpread2({}, csvDefaultOptions), exportCSV); // get data for csv export

        let data;

        if (typeof source !== 'undefined') {
          data = source;
        } else if (options.exportAll) {
          data = this.props.data;
        } else if (options.onlyExportFiltered) {
          const payload = {};
          this.tableExposedAPIEmitter.emit('get.filtered.rows', payload);
          data = payload.result;
        } else {
          const payload = {};
          this.tableExposedAPIEmitter.emit('get.table.data', payload);
          data = payload.result;
        } // filter data by row selection


        if (options.onlyExportSelection) {
          const payload = {};
          this.tableExposedAPIEmitter.emit('get.selected.rows', payload);
          const selections = payload.result;
          data = data.filter(row => !!selections.find(sel => row[keyField] === sel));
        }

        const content = transform(data, meta, columns, this._, options);
        save(content, options);
      };
    }

  }, _temp;
});

const Operation = {
  csvOperation
};

var statelessDecorator = (Base => {
  var _temp;

  return _temp = class StatelessOperation extends Operation.csvOperation(Base) {
    constructor(...args) {
      super(...args);

      this.registerExposedAPI = tableExposedAPIEmitter => {
        this.tableExposedAPIEmitter = tableExposedAPIEmitter;
      };
    }

  }, _temp;
});

/* eslint-disable react/destructuring-assignment */
var createSearchContext = ((options = {
  searchFormatted: false,
  afterSearch: null,
  onColumnMatch: null
}) => (_, isRemoteSearch, handleRemoteSearchChange) => {
  const SearchContext = /*#__PURE__*/React__default['default'].createContext();

  class SearchProvider extends React__default['default'].Component {
    constructor(_props) {
      super(_props);

      this.triggerListener = (result, skipInit) => {
        const {
          dataChangeListener
        } = this.props;

        if (options.afterSearch && !skipInit) {
          options.afterSearch(result);
        }

        if (dataChangeListener) {
          dataChangeListener.emit('filterChanged', result.length);
        }
      };

      this.search = (props = this.props) => {
        const {
          data,
          columns
        } = props;
        const searchText = props.searchText.toLowerCase();
        return data.filter((row, ridx) => {
          for (let cidx = 0; cidx < columns.length; cidx += 1) {
            const column = columns[cidx];
            if (column.searchable === false) continue;

            let targetValue = _.get(row, column.dataField);

            if (column.formatter && options.searchFormatted) {
              targetValue = column.formatter(targetValue, row, ridx, column.formatExtraData);
            } else if (column.filterValue) {
              targetValue = column.filterValue(targetValue, row);
            }

            if (options.onColumnMatch) {
              if (options.onColumnMatch({
                searchText,
                value: targetValue,
                column,
                row
              })) {
                return true;
              }
            } else {
              if (targetValue !== null && typeof targetValue !== 'undefined') {
                targetValue = targetValue.toString().toLowerCase();

                if (targetValue.indexOf(searchText) > -1) {
                  return true;
                }
              }
            }
          }

          return false;
        });
      };

      const {
        searchText: _searchText
      } = _props;
      let initialData = _props.data;

      if (isRemoteSearch() && _searchText !== '') {
        handleRemoteSearchChange(_searchText);
      } else {
        initialData = this.search(_props);
        this.triggerListener(initialData, true);
      }

      this.state = {
        data: initialData
      };
    }

    getSnapshotBeforeUpdate(prevProps) {
      if (isRemoteSearch()) {
        if (prevProps.searchText !== this.props.searchText) {
          return 'handleRemoteSearchChange';
        }

        if (!_.isEqual(prevProps.data, this.props.data)) {
          return 'setState';
        }

        return null;
      }

      if (prevProps.searchText !== this.props.searchText) {
        return 'triggerListener';
      }

      if (!_.isEqual(prevProps.data, this.props.data)) {
        return 'triggerListener';
      }

      return null;
    } // eslint-disable-next-line no-unused-vars


    componentDidUpdate(_prevProps, _prevState, snapshot) {
      // static getDerivedStateFromProps(prevProps) {
      if (snapshot !== null) {
        // eslint-disable-next-line default-case
        switch (snapshot) {
          case 'handleRemoteSearchChange':
            handleRemoteSearchChange(this.props.searchText);
            break;

          case 'triggerListener':
            // eslint-disable-next-line no-case-declarations
            const result = this.search(this.props);
            this.triggerListener(result);
            this.setState({
              data: result
            });
            break;

          case 'setState':
            this.setState({
              data: this.props.data
            });
            break;
        }
      } // if (prevProps.searchText !== this.props.searchText) {
      //   if (isRemoteSearch()) {
      //     handleRemoteSearchChange(this.props.searchText);
      //   } else {
      //     const result = this.search(this.props);
      //     this.triggerListener(result);
      //     this.setState({
      //       data: result
      //     });
      //   }
      // } else {
      //   if (isRemoteSearch()) {
      //     this.setState({ data: this.props.data });
      //   } else if (!_.isEqual(this.props.data, prevProps.data)) {
      //     const result = this.search(this.props);
      //     this.triggerListener(result);
      //     this.setState({
      //       data: result
      //     });
      //   }
      // }

    }

    getSearched() {
      const {
        data
      } = this.state;
      return data;
    }

    render() {
      const {
        data
      } = this.state;
      const {
        children
      } = this.props;
      return /*#__PURE__*/React__default['default'].createElement(SearchContext.Provider, {
        value: {
          data
        }
      }, children);
    }

  }

  SearchProvider.propTypes = {
    data: PropTypes__default['default'].array.isRequired,
    // eslint-disable-next-line react/no-unused-prop-types
    columns: PropTypes__default['default'].array.isRequired,
    searchText: PropTypes__default['default'].string,
    dataChangeListener: PropTypes__default['default'].object
  };
  return {
    Provider: SearchProvider,
    Consumer: SearchContext.Consumer
  };
});

const ToolkitContext = /*#__PURE__*/React__default['default'].createContext();

class ToolkitProvider extends statelessDecorator(React__default['default'].Component) {
  constructor(...args) {
    var _this$props$columns$r, _this$props, _this$props$columns;

    super(...args);
    this._ = null;
    this.state = {
      searchText: typeof this.props.search === 'object' ? this.props.search.defaultSearch || '' : '',
      columnToggle: (_this$props$columns$r = (_this$props = this.props) === null || _this$props === void 0 ? void 0 : (_this$props$columns = _this$props.columns) === null || _this$props$columns === void 0 ? void 0 : _this$props$columns.reduce((obj, column) => {
        obj[column.dataField] = !column.hidden;
        return obj;
      }, {})) !== null && _this$props$columns$r !== void 0 ? _this$props$columns$r : null
    };

    this.onSearch = searchText => {
      if (searchText !== this.state.searchText) {
        this.setState({
          searchText
        });
      }
    };

    this.onClear = () => {
      this.setState({
        searchText: ''
      });
    };

    this.onColumnToggle = (dataField, forceValue) => {
      var _this$props$columns$f, _this$props$columns2;

      const selectedColumn = (_this$props$columns$f = (_this$props$columns2 = this.props.columns) === null || _this$props$columns2 === void 0 ? void 0 : _this$props$columns2.find(column => column.dataField === dataField)) !== null && _this$props$columns$f !== void 0 ? _this$props$columns$f : null;
      this.setState(({
        columnToggle
      }) => {
        var _ref, _columnToggle$dataFie;

        const fallbackValue = (_ref = (_columnToggle$dataFie = !(columnToggle !== null && columnToggle !== void 0 && columnToggle[dataField])) !== null && _columnToggle$dataFie !== void 0 ? _columnToggle$dataFie : selectedColumn === null || selectedColumn === void 0 ? void 0 : selectedColumn.hidden) !== null && _ref !== void 0 ? _ref : false;
        const value = typeof forceValue === 'boolean' ? forceValue : fallbackValue;
        return {
          columnToggle: _objectSpread2(_objectSpread2({}, columnToggle), {}, {
            [dataField]: value
          })
        };
      });
    };

    this.setDependencyModules = _ => {
      this._ = _;
    };
  }

  componentDidUpdate(prevProps, prevState) {
    var _this$props2;

    if ((prevProps === null || prevProps === void 0 ? void 0 : prevProps.columnToggle) !== ((_this$props2 = this.props) === null || _this$props2 === void 0 ? void 0 : _this$props2.columnToggle)) {
      if (!this._.isEqual(prevState.columnToggle, this.state.columnToggle)) {
        var _this$props$columns$r2, _this$props3, _this$props3$columns;

        const columnToggle = (_this$props$columns$r2 = (_this$props3 = this.props) === null || _this$props3 === void 0 ? void 0 : (_this$props3$columns = _this$props3.columns) === null || _this$props3$columns === void 0 ? void 0 : _this$props3$columns.reduce((obj, column) => {
          obj[column.dataField] = !column.hidden;
          return obj;
        }, {})) !== null && _this$props$columns$r2 !== void 0 ? _this$props$columns$r2 : null; // eslint-disable-next-line react/no-did-update-set-state

        this.setState({
          columnToggle
        });
      }
    }
  }

  render() {
    const baseProps = {
      keyField: this.props.keyField,
      columns: this.props.columns,
      data: this.props.data,
      bootstrap4: this.props.bootstrap4,
      setDependencyModules: this.setDependencyModules,
      registerExposedAPI: this.registerExposedAPI
    };

    if (this.props.search) {
      baseProps.search = {
        searchContext: createSearchContext(this.props.search),
        searchText: this.state.searchText
      };
    }

    if (this.props.columnToggle) {
      baseProps.columnToggle = {
        toggles: this.state.columnToggle
      };
    }

    return /*#__PURE__*/React__default['default'].createElement(ToolkitContext.Provider, {
      value: {
        searchProps: {
          searchText: this.state.searchText,
          onSearch: this.onSearch,
          onClear: this.onClear
        },
        csvProps: {
          onExport: this.handleExportCSV
        },
        columnToggleProps: {
          columns: this.props.columns,
          toggles: this.state.columnToggle,
          onColumnToggle: this.onColumnToggle
        },
        baseProps
      }
    }, this.props.children);
  }

}

ToolkitProvider.propTypes = {
  keyField: PropTypes__default['default'].string.isRequired,
  data: PropTypes__default['default'].array.isRequired,
  columns: PropTypes__default['default'].array.isRequired,
  children: PropTypes__default['default'].node.isRequired,
  bootstrap4: PropTypes__default['default'].bool,
  search: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].shape({
    defaultSearch: PropTypes__default['default'].string,
    searchFormatted: PropTypes__default['default'].bool
  })]),
  exportCSV: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].shape({
    fileName: PropTypes__default['default'].string,
    separator: PropTypes__default['default'].string,
    ignoreHeader: PropTypes__default['default'].bool,
    ignoreFooter: PropTypes__default['default'].bool,
    noAutoBOM: PropTypes__default['default'].bool,
    blobType: PropTypes__default['default'].string,
    exportAll: PropTypes__default['default'].bool,
    onlyExportFiltered: PropTypes__default['default'].bool,
    onlyExportSelection: PropTypes__default['default'].bool
  })])
};
ToolkitProvider.defaultProps = {
  search: false,
  exportCSV: false,
  bootstrap4: true
};
var Context = {
  Provider: ToolkitProvider,
  Consumer: ToolkitContext.Consumer
};

/* eslint-disable react/jsx-props-no-spreading */

const Toolkitprovider = props => /*#__PURE__*/React__default['default'].createElement(Context.Provider, props, /*#__PURE__*/React__default['default'].createElement(Context.Consumer, null, tookKitProps => props.children(tookKitProps)));

Toolkitprovider.propTypes = {
  children: PropTypes__default['default'].func.isRequired
};

/* eslint-disable prefer-rest-params */

function handleDebounce(func, wait, immediate) {
  let timeout;
  return () => {
    const later = () => {
      timeout = null;

      if (!immediate) {
        func.apply(this, arguments);
      }
    };

    const callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait || 0);

    if (callNow) {
      func.appy(this, arguments);
    }
  };
}

class SearchBar extends React__default['default'].Component {
  constructor(props) {
    super(props);

    this.onChangeValue = e => {
      this.setState({
        value: e.target.value
      });
    };

    this.onKeyup = () => {
      const {
        delay,
        onSearch
      } = this.props;
      const debounceCallback = handleDebounce(() => {
        var _this$inputRef$curren, _this$inputRef$curren2;

        onSearch((_this$inputRef$curren = (_this$inputRef$curren2 = this.inputRef.current) === null || _this$inputRef$curren2 === void 0 ? void 0 : _this$inputRef$curren2.value) !== null && _this$inputRef$curren !== void 0 ? _this$inputRef$curren : '');
      }, delay);
      debounceCallback();
    };

    this.state = {
      value: props.searchText
    };
    this.inputRef = /*#__PURE__*/React.createRef();
  } // UNSAFE_componentWillReceiveProps(nextProps) {


  static getDerivedStateFromProps(props, state) {
    if (props.searchText !== state.searchText) {
      return {
        value: props.searchText
      };
    }

    return null;
  }

  render() {
    const {
      className,
      style,
      placeholder,
      tableId,
      srText
    } = this.props;
    const {
      value
    } = this.state;
    return /*#__PURE__*/React__default['default'].createElement("label", {
      htmlFor: `search-bar-${tableId}`,
      className: "search-label"
    }, /*#__PURE__*/React__default['default'].createElement("span", {
      id: `search-bar-${tableId}-label`,
      className: "sr-only"
    }, srText), /*#__PURE__*/React__default['default'].createElement("input", {
      ref: this.inputRef,
      id: `search-bar-${tableId}`,
      type: "text",
      style: style,
      "aria-labelledby": `search-bar-${tableId}-label`,
      onKeyUp: () => this.onKeyup(),
      onChange: this.onChangeValue,
      className: `form-control ${className}`,
      value: value,
      placeholder: placeholder || SearchBar.defaultProps.placeholder
    }));
  }

}

SearchBar.propTypes = {
  onSearch: PropTypes__default['default'].func.isRequired,
  className: PropTypes__default['default'].string,
  placeholder: PropTypes__default['default'].string,
  style: PropTypes__default['default'].object,
  delay: PropTypes__default['default'].number,
  searchText: PropTypes__default['default'].string,
  tableId: PropTypes__default['default'].string,
  srText: PropTypes__default['default'].string
};
SearchBar.defaultProps = {
  className: '',
  style: {},
  placeholder: 'Search',
  delay: 250,
  searchText: '',
  tableId: '0',
  srText: 'Search this table'
};

/* eslint-disable react/button-has-type */

const ClearButton = ({
  onClear,
  text,
  className
}) => /*#__PURE__*/React__default['default'].createElement("button", {
  className: `btn btn-default ${className}`,
  onClick: onClear
}, text);

ClearButton.propTypes = {
  onClear: PropTypes__default['default'].func.isRequired,
  className: PropTypes__default['default'].string,
  text: PropTypes__default['default'].string
};
ClearButton.defaultProps = {
  text: 'Clear',
  className: ''
};

const Search = {
  SearchBar,
  ClearSearchButton: ClearButton
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = _objectWithoutPropertiesLoose(source, excluded);
  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

const ExportCSVButton = props => {
  const {
    onExport,
    children,
    className
  } = props,
        rest = _objectWithoutProperties(props, ["onExport", "children", "className"]);

  return /*#__PURE__*/React__default['default'].createElement("button", _extends({
    type: "button",
    className: `react-bs-table-csv-btn btn btn-default ${className}`,
    onClick: () => onExport()
  }, rest), children);
};

ExportCSVButton.propTypes = {
  children: PropTypes__default['default'].node.isRequired,
  onExport: PropTypes__default['default'].func.isRequired,
  className: PropTypes__default['default'].string,
  style: PropTypes__default['default'].object
};
ExportCSVButton.defaultProps = {
  className: '',
  style: {}
};

const CSVExport = {
  ExportCSVButton
};

const ToggleList = ({
  columns,
  onColumnToggle,
  toggles,
  contextual,
  className,
  btnClassName
}) => /*#__PURE__*/React__default['default'].createElement("div", {
  className: `btn-group btn-group-toggle ${className}`,
  "data-toggle": "buttons"
}, columns.map(column => _objectSpread2(_objectSpread2({}, column), {}, {
  toggle: toggles[column.dataField]
})).map(column => /*#__PURE__*/React__default['default'].createElement("button", {
  type: "button",
  key: column.dataField,
  className: `${btnClassName} btn btn-${contextual} ${column.toggle ? 'active' : ''}`,
  "data-toggle": "button",
  "aria-pressed": column.toggle ? 'true' : 'false',
  onClick: () => onColumnToggle(column.dataField)
}, column.text)));

ToggleList.propTypes = {
  columns: PropTypes__default['default'].array.isRequired,
  toggles: PropTypes__default['default'].object.isRequired,
  onColumnToggle: PropTypes__default['default'].func.isRequired,
  btnClassName: PropTypes__default['default'].string,
  className: PropTypes__default['default'].string,
  contextual: PropTypes__default['default'].string
};
ToggleList.defaultProps = {
  btnClassName: '',
  className: '',
  contextual: 'primary'
};

const ColumnToggle = {
  ToggleList
};

const ToolkitContext$1 = Context;

exports.CSVExport = CSVExport;
exports.ColumnToggle = ColumnToggle;
exports.Search = Search;
exports.ToolkitContext = ToolkitContext$1;
exports.default = Toolkitprovider;
